package pps.core.system.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.core.common.session.PpsUserSession;
import pps.core.system.config.LoginConfig;
import pps.core.system.entity.SysRoleEnt;
import pps.core.system.entity.SysRoleView;
import pps.core.system.entity.SysUserRoleRelEnt;
import pps.core.system.mapper.SysRoleMapper;
import pps.core.system.mapper.SysRoleViewMapper;
import pps.core.system.mapper.SysUserRoleRelMapper;
import pps.core.system.service.data.sys_role.*;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.exception.XServiceException;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.helper.XTransactionHelper;

import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

@XText("角色管理")
@XService
public class SysRoleService {

    @XText("查询角色列表")
    @XApiPost
    public XPageResult<QuerySysRoleViewOutput> selectSysRoleViewPage(XContext context, QuerySysRoleViewInput input) {
        return XPageResult.success(Collections.emptyList(), 1, 1, 0);
    }

    @XText("新增角色")
    @XApiPost
    public XServiceResult createSysRole(XContext context, CreateSysRoleInput input) throws XServiceException {
        return XTransactionHelper.begin(context, () -> {
            SysRoleEnt entity = new SysRoleEnt();
            SysRoleMapper mapper = context.getBean(SysRoleMapper.class);
            QueryWrapper<SysRoleEnt> wrapper = new QueryWrapper<>();
            wrapper.lambda().eq(SysRoleEnt::getRoleName, input.getRoleName());
            if (CharSequenceUtil.isNotBlank(input.getOuId())) {
                wrapper.lambda().eq(SysRoleEnt::getOuId, input.getOuId());
            }
            List<SysRoleEnt> sysRoleEnts = mapper.selectList(wrapper);
            if (CollUtil.isNotEmpty(sysRoleEnts)) {
                return XServiceResult.error(200, "角色名称已存在!");
            }
            XCopyUtils.copyObject(input, entity);
            String roleId = UUID.randomUUID().toString();
            entity.setId(roleId);
            entity.setCreateTime(new Date());
            PpsUserSession session = context.getSession(PpsUserSession.class);
            entity.setCreateByName(session.getUserName());
            entity.setCreateById(session.getId());
            //用户不选的话需要绑定为当前操作员的组织机构
            if (CharSequenceUtil.isBlank(entity.getOuId())) {
                entity.setOuId(session.getOuId());
            }
            try {
                mapper.insert(entity);
            } catch (Exception e) {
                throw new XServiceException(e);
            }
            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });
    }

    @XText("修改角色信息")
    @XApiPost
    public XServiceResult updateSysRole(XContext context, UpdateSysRoleInput input) throws XServiceException {
        return XTransactionHelper.begin(context, () -> {
            SysRoleMapper mapper = context.getBean(SysRoleMapper.class);
            SysRoleEnt entity = mapper.selectOne(new QueryWrapper<SysRoleEnt>()
                    .lambda()
                    .eq(SysRoleEnt::getId, input.getId()));
            if (ObjectUtil.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            if (!CharSequenceUtil.equals(entity.getRoleName(), input.getRoleName())) {
                QueryWrapper<SysRoleEnt> wrapper = new QueryWrapper<>();
                wrapper.lambda().eq(SysRoleEnt::getRoleName, input.getRoleName());
                if (CharSequenceUtil.isNotBlank(input.getOuId())) {
                    wrapper.lambda().eq(SysRoleEnt::getOuId, input.getOuId());
                }
                List<SysRoleEnt> sysRoleEnts = mapper.selectList(wrapper);
                if (CollUtil.isNotEmpty(sysRoleEnts)) {
                    return XServiceResult.error(200, "角色名称已存在!");
                }
            }
            XCopyUtils.copyObject(input, entity);
            PpsUserSession session = context.getSession(PpsUserSession.class);
            entity.setModifyTime(new Date());
            entity.setModifyByName(session.getUserName());
            entity.setModifyById(session.getId());
            mapper.updateById(entity);
            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });
    }

    @XText("删除角色")
    @XApiPost
    public XServiceResult deleteSysRole(XContext context, DeleteSysRoleInput input) throws XServiceException {
        return XTransactionHelper.begin(context, () -> {
            SysRoleMapper mapper = context.getBean(SysRoleMapper.class);
            SysUserRoleRelMapper relMapper = context.getBean(SysUserRoleRelMapper.class);
            SysRoleEnt entity = mapper.selectOne(new QueryWrapper<SysRoleEnt>()
                    .lambda()
                    .eq(SysRoleEnt::getId, input.getId()));
            if (ObjectUtil.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            String value = LoginConfig.adminRoles.findValue(context);
            if (CharSequenceUtil.isNotBlank(value) && CharSequenceUtil.contains(value, entity.getId())) {
                return XServiceResult.error(200, "此角色是管理员角色,请勿删除!");
            }
            List<SysUserRoleRelEnt> relEnts = relMapper.selectList(new QueryWrapper<SysUserRoleRelEnt>()
                    .lambda()
                    .eq(SysUserRoleRelEnt::getRoleId, entity.getId()));
            if (CollUtil.isNotEmpty(relEnts)) {
                List<String> collect = relEnts.stream().map(SysUserRoleRelEnt::getId).collect(Collectors.toList());
                relMapper.deleteBatchIds(collect);
            }
            mapper.deleteById(entity);
            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });
    }

    @XText("根据组织机构id获取角色列表")
    @XApiPost
    public XPageResult<QuerySysRoleViewOutput> getRoleListByOuIdPage(XContext context, GetRoleListByOuIdInput input) {
        SysRoleViewMapper viewMapper = context.getBean(SysRoleViewMapper.class);
        SysRoleView record = new SysRoleView();
        XCopyUtils.copyObject(input, record);
        if (ObjectUtil.isNotNull(input.getShowAll()) && input.getShowAll()) {
            record.setOuId(null);
        }
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SysRoleView> list = viewMapper.selectSysRoleListByOuId(record);
        PageInfo<SysRoleView> pageInfo = new PageInfo<>(list);
        List<QuerySysRoleViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysRoleViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }
}